      program test_sig99
      implicit none
      include 'vector_eos.dek'

 
! tests the opacity routines
! 
! ionmax  = number of isotopes in the network
! xmass   = mass fractions
! ymass   = molar fractions
! aion    = number of nucleons
! zion    = number of protons

      integer, parameter :: ionmax = 3
      real*8             :: xmass(ionmax),ymass(ionmax), &
                            aion(ionmax),zion(ionmax),wion(ionmax), &
                            temp,den,abar,zbar,wbar,ye,nxcess, &
                            pin,xin,etain,orad,ocond,opac,sigrad,sigcond,sigma

! a format statement
01    format(1x,a6,t10,a6,1pe10.3,t30,a6,1pe10.3,t50,a6,1pe10.3)


! set the mass fractions, z's and a's of the composition
! hydrogen
      xmass(1) = 0.75d0
      aion(1)  = 1.0d0
      zion(1)  = 1.0d0
      wion(1)  = aion(1)

! helium
      xmass(2) = 0.23d0
      aion(2)  = 4.0d0
      zion(2)  = 2.0d0
      wion(2)  = aion(2)

! carbon 12
      xmass(3) = 0.02d0
      aion(3)  = 12.0d0
      zion(3)  = 6.0d0
      wion(3)  = aion(3)


! get abar, zbar and a few other composition variables
      call azbar(xmass,aion,zion,wion,ionmax, &
                 ymass,abar,zbar,wbar,ye,nxcess)

! set the input vector
      temp_row(1) = 1.0d8
      den_row(1)  = 1.0d6
      abar_row(1) = abar
      zbar_row(1) = zbar

! here the eos pipeline is only 1 element long
      jlo_eos = 1
      jhi_eos = 1


! call an equation of state
      call read_helm_table
      call helmeos

! load the electron-positron pressure, number density, and chemcial potential
       pin = pele_row(1) + ppos_row(1)
       xin = xne_row(1) + xnp_row(1)
       etain = etaele_row(1)


! get the opacities and conductivities
       call sig99(temp_row(1),den_row(1),xmass,zion,aion,ionmax, &
                  pin,xin,etain, &
                  orad,ocond,opac, &
                  sigrad,sigcond,sigma)

! write out what we got
      write(6,01) 'sig99 ','orad =',orad
      write(6,01) '      ','ocond=',ocond
      write(6,01) '      ','opac =',opac

      write(6,*)
      write(6,01) '      ','srad =',sigrad
      write(6,01) '      ','scond=',sigcond
      write(6,01) '      ','sigma=',sigma

      write(6,*) ' '

      stop 'normal termination'
      end program test_sig99



      include 'azbar.f90'
      include 'kapbar.f90'
      include 'sig99.f90'
      include 'helmholtz_for_public.f90'
